# -*- coding: UTF-8 -*-

import urllib3
from urllib.parse import quote
import sys
import io
import json
import requests
import hashlib
import random
from enum import Enum

ERROR_DICT = {
    '1001': 'Translate API: Unsupported language type',
    '1002': 'Translate API: Text too long',
    '1003': 'Translate API: Invalid PID',
    '1004': 'Translate API: Trial PID limit reached',
    '1005': 'Translate API: PID traffic too high',
    '1006': 'Translate API: Insufficient balance',
    '1007': 'Translate API: Random number does not exist',
    '1008': 'Translate API: Signature does not exist',
    '1009': 'Translate API: The signature is incorrect',
    '10010': 'Translate API: Text does not exist',
    '1050': 'Translate API: Internal server error',
}


class SogouTranslateException(Exception):
    def __init__(self, message):

        super().__init__(message)


def _error_code_to_exception(code: str) -> SogouTranslateException:
    return SogouTranslateException(ERROR_DICT[code])


class SogouLanguages(Enum):
    AR = 'ar'  # Arabic
    ET = 'et'  # Estonian
    BG = 'bg'  # Bulgarian
    PL = 'pl'  # Polish
    KO = 'ko'  # Korean
    BS_LATN = 'bs-Latn'  # Bosnian (Latin)
    FA = 'fa'  # Persian
    MWW = 'mww'  # Hmong Daw
    DA = 'da'  # Danish
    DE = 'de'  # German
    RU = 'ru'  # Russian
    FR = 'fr'  # French
    FI = 'fi'  # Finnish
    TLH_QAAK = 'tlh-Qaak'  # Klingon (pIqaD)
    TLH = 'tlh'  # Klingon
    HR = 'hr'  # Croatian
    OTQ = 'otq'  # Querétaro Otomi
    CA = 'ca'  # Catalan
    CS = 'cs'  # Czech
    RO = 'ro'  # Romanian
    LV = 'lv'  # Latvian
    HT = 'ht'  # Haitian Creole
    LT = 'lt'  # Lithuanian
    NL = 'nl'  # Dutch
    MS = 'ms'  # Malay
    MT = 'mt'  # Maltese
    PT = 'pt'  # Portuguese
    JA = 'ja'  # Japanese
    SL = 'sl'  # Slovenian
    TH = 'th'  # Thai
    TR = 'tr'  # Turkish
    SR_LATN = 'sr-Latn'  # Serbian (Latin)
    SR_CYRL = 'sr-Cyrl'  # Serbian (Cyrillic)
    SK = 'sk'  # Slovak
    SW = 'sw'  # Kiswahili
    AF = 'af'  # South African Common Dutch
    NO = 'no'  # Norwegian
    EN = 'en'  # English
    ES = 'es'  # Spanish
    UK = 'uk'  # Ukrainian
    UR = 'ur'  # Urdu
    EL = 'el'  # Greek
    HU = 'hu'  # Hungarian
    CY = 'cy'  # Welsh
    YUA = 'yua'  # Yucatec Maya
    HE = 'he'  # Hebrew
    ZH_CHS = 'zh-CHS'  # Chinese Simplified
    IT = 'it'  # Italian
    HI = 'hi'  # Hindi
    ID = 'id'  # Indonesian
    ZH_CHT = 'zh-CHT'  # Chinese Traditional
    VI = 'vi'  # Vietnamese
    SV = 'sv'  # Swedish
    YUE = 'yue'  # Cantonese
    FJ = 'fj'  # fijian
    FIL = 'fil'  # Filipino
    SM = 'sm'  # Samoan language
    TO = 'to'  # lea fakatonga
    TY = 'ty'  # Tahiti language
    MG = 'mg'  # Malagasy language
    BN = 'bn'  # Bengali


class SogouTranslate:

    SOGOU_API_URL = 'https://fanyi.sogou.com/reventondc/api/sogouTranslate'

    def __init__(self, pid: str, secret_key: str):
        """Initialize SogouTranslate

        Arguments:
            pid {str} -- pid for this service
            secret_key {str} -- secret key for this server

        Raises:
            SogouTranslateException -- while exeception occurs calling this service, detailed exception message will be provided.
        """

        if (not pid) or (not secret_key):
            raise SogouTranslateException('pid or secret key cannot be empty')

        self.pid = pid
        self.secret_key = secret_key

    def _generate_salt(self) -> str:
        """Generate salt string

        Returns:
            str -- the salt string
        """

        return hashlib.sha256(str(random.getrandbits(256)).encode('utf-8')).hexdigest()[:19]

    def _compute_sign(self, source_text: str, salt: str) -> str:
        """Compute the sign string according to Sogou's requirement (https://deepi.sogou.com/docs/fanyiDoc)

        Arguments:
            source_text {str} -- The text to be translated
            salt {str} -- the salt string

        Returns:
            [str] -- The sign string
        """

        text = self.pid + source_text + salt + self.secret_key
        return hashlib.md5(text.encode('utf-8')).hexdigest()

    def _generate_data(self, source_text: str, from_language: SogouLanguages,
                       to_language: SogouLanguages):
        """Generate the data for requesting the translating service

        Arguments:
            source_text {str} -- the text to be translated
            from_language {SogouLanguages} -- the language type of source_text
            to_language {SogouLanguages} -- the language for translation

        Returns:
            dict -- a dictionary containing the data to be posted to Sogou's API server
        """

        salt = self._generate_salt()
        data = {
            'q': source_text,  # text
            'from': from_language.value,  # from language
            'to': to_language.value,  # to language
            'pid': self.pid,  # pid
            'salt': salt,  # salt
            'sign': self._compute_sign(source_text, salt),  # sign
            'charset': 'utf-8',  # charset
            #     'callback': '', # optional for CORs
        }
        return data

    def translate(self, source_text: str, from_language: SogouLanguages,
                  to_language: SogouLanguages) -> str:
        """The translate API

        Arguments:
            source_text {str} -- the text to be translated
            from_language {SogouLanguages} -- the source language type
            to_language {SogouLanguages} -- the target lanaguage type

        Raises:
            SogouTranslateException -- while exeception occurs calling this service, detailed exception message will be provided

        Returns:
            [str] -- the translated text
        """
        if not source_text:
            raise SogouTranslateException('Source text does not exist')

        data = self._generate_data(
            source_text, from_language, to_language
        )
        res = requests.post(self.SOGOU_API_URL, data=data)

        if not res.ok:
            raise SogouTranslateException(
                'Translation request is not successful'
            )

        json_res = json.loads(res.text)

        error_code = json_res['errorCode']
        if error_code != '0':
            raise _error_code_to_exception(error_code)

        return json_res['translation']
		
sys.stdout=io.TextIOWrapper(sys.stdout.buffer,encoding='utf8')
http = urllib3.PoolManager()
x = quote(sys.argv[1])
y = "http://fanyi.youdao.com/openapi.do?keyfrom=SkyHacker&key=25021457&type=data&doctype=json&version=1.2&q="
r = http.request('GET', y+x)
r = r.data.decode('utf-8')
content = '<div id="json">'+r+'</div><div class="ydwrap"><div id="exp_w"><p>有道基本释义</p><div id="explains"></div></div><div id="web_w"><p>有道网络释义</p><div id="web"></div></div></div>'

trans = SogouTranslate('923dd4e8c91dc56f9714ba204aaa6377', '76ff365f8c6209414a40ae94222599c7')
if sys.argv[1] >= u'\u4e00' and sys.argv[1]<=u'\u9fa5': 
   sg_trans = trans.translate(sys.argv[1],from_language=SogouLanguages.ZH_CHS, to_language=SogouLanguages.EN)
else:
   sg_trans = trans.translate(sys.argv[1],from_language=SogouLanguages.EN, to_language=SogouLanguages.ZH_CHS)
content += '<div class="ydwrap"><div id="gg_w"><p>搜狗翻译</p><span>'+sys.argv[1]+'<hr><p>'+sg_trans+'</p></span></div></div>'

head = """
<style>
#json{
display:none;
}
.ydwrap{
font-family:"Microsoft YaHei UI",tahoma;
font-size:14px;
line-height:22px;
margin-left:10px;
}
.ydwrap p{
margin:5px 0;
font-size:15px;
color: orangered;


}
.ydwrap div{
margin-bottom:12px;
}
</style>
<script>
window.onload = function(){
var json = document.getElementById("json").innerHTML;
var exp = document.getElementById("explains");
var web = document.getElementById("web");
var exp_w = document.getElementById("exp_w");
var web_w = document.getElementById("web_w");
var gg_w = document.getElementById("gg_w");
var obj=eval("("+json+")");
var explains="",webc="",a;
if(obj.basic){
if(obj.basic.explains){
var arr1 = obj.basic.explains;
for (a = 0; a < arr1.length; a++) {
	explains += arr1[a]+"<br/>";
}
exp.innerHTML = explains;
}
}
if(obj.web){
var arr2 = obj.web[0].value;
for (a = 0; a < arr2.length; a++) {
	webc += arr2[a]+"<br/>";
}
web.innerHTML = webc;
}
if(!exp.innerText){
exp_w.style.display="none";
}
if(!web.innerText){
web_w.style.display="none";
}
}
</script>"""

print('<head>\n%s\n</head>\n%s' % (head, content))
